#include <qpopupmenu.h>
#include "chattab.h"
#include "icons.h"
#include "global.h"

ChatTab::ChatTab(QWidget *parent, const char *name): QTabWidget(parent, name), prevdel(NULL), board(NULL)
{
#ifndef WIN32
    setTabShape(QTabWidget::Triangular);
#endif
    setFocusPolicy(QWidget::NoFocus);
    connect(this, SIGNAL(currentChanged(QWidget *)), this, SLOT(tabChanged(QWidget *)));
}

ChatTab::~ChatTab()
{
}

bool ChatTab::exists(const QString &name, bool cs)
{
    ChannelMap::iterator it;
    if(!cs)
    {
	for (it = cmap.begin(); it != cmap.end(); it++ )
	if (it.key().lower() == name.lower() && !it.data()->isBoard())
	    return true;
    }
    else
	return cmap.contains(name);
    return false;
}

bool ChatTab::isPrivate(const QString &name)
{
    ChannelMap::iterator it = cmap.find(name);
    return (it != cmap.end()) ? it.data()->isPrivate() : false;
}

bool ChatTab::addChannel(const QString &name, QColor bcolor, QFont chatfont)
{
    QString n = (name[0]=='#') ?  name : "#" + name;
    
    if (exists(n)) return false;
    Channel *c = new Channel(this, n, bcolor, chatfont);
    
    addTab(c, Icons::icon_channel_normal, n);
    cmap[n] = c;
    showPage(c);
    
    connect(c, SIGNAL(activityDetected()), this, SIGNAL(actDetected()));
    connect(c, SIGNAL(doubleClicked(int, int)), this, SLOT(onChannelDoubleClicked(int, int)));
    connect(c, SIGNAL(leave(const QString&)), this, SIGNAL(leave(const QString&)));
    connect(c, SIGNAL(keypressed(const QString &)), this, SIGNAL(keypressed(const QString &)));    
    connect(c, SIGNAL(sig_topic_clear(const QString &)), this, SIGNAL(sig_topic_clear(const QString &)));
    connect(c, SIGNAL(linkClicked(const QString &)), this, SLOT(on_linkClicked(const QString &)));
    
    return true;
}

void ChatTab::on_linkClicked(const QString &link)
{
    Channel *c = (Channel*) sender();
    emit sig_linkClicked(link, c->name());
}

bool ChatTab::addBoard(QFont chatfont)
{
    QString tmp;
    
    if (board) return false;
    
    tmp = MAIN_BOARD;
    do 
	    tmp.prepend("Main");
    while (exists(tmp));
    
    Channel *c = new Channel(this, tmp, QColor("#FFFFFF"), chatfont, false, true);
    
    board = c;

    addTab(c, Icons::icon_board_normal, tr("Board"));
    cmap[tmp] = c;
    showPage(c);

    connect(c, SIGNAL(activityDetected()), this, SIGNAL(actDetected()));
//    connect(c, SIGNAL(doubleClicked(int, int)), this, SLOT(onChannelDoubleClicked(int, int)));
//    connect(c, SIGNAL(keypressed(const QString &)), this, SIGNAL(keypressed(const QString &)));
    connect(c, SIGNAL(linkClicked(const QString &)), this, SLOT(on_linkClicked(const QString &)));
    
    return true;
}

bool ChatTab::delBoard()
{
    if (board)
    {
	board = NULL;
    
	for (ChannelMap::iterator it = cmap.begin(); it != cmap.end(); it++)
	    if (it.data()->isBoard())
		return del(it.key());
    }
    return false;
}

void ChatTab::onChannelDoubleClicked(int para, int pos)
{
    Channel *c = (Channel*)sender();
    c->removeSelection();
    emit channelDoubleClicked(c->name(), para, pos);
}

bool ChatTab::del(const QString &name)
{
    if (prevdel)
    {
	delete prevdel;
	prevdel = NULL;
    }
    
    ChannelMap::iterator it = cmap.find(name);
    if (it != cmap.end())
    {
	prevdel = it.data();
	removePage(it.data());
	cmap.remove(it);
	return true;
    }
    return false;
}

bool ChatTab::delChannel(const QString &name)
{
    return del((name[0]=='#') ?  name : "#" + name);
}

bool ChatTab::renamePrivate(const QString &from, const QString &to)
{
    if (exists(to, true)) return false;  
    
    ChannelMap::iterator it = cmap.find(from);
    if (it != cmap.end())
    {
	Channel *c = it.data();
	cmap[to] = c;
	setTabLabel(c, to);
	cmap.remove(it);
	return true;
    }
    return false;
}

bool ChatTab::addPrivate(const QString &name, QColor bcolor, QFont chatfont)
{
    if (exists(name, true)) return false;
    
    Channel *c = new Channel(this, name, bcolor, chatfont, true);
    addTab(c, Icons::icon_private, name);
    cmap[name] = c;
    showPage(c);
    
    connect(c, SIGNAL(activityDetected()), this, SIGNAL(actDetected()));
    connect(c, SIGNAL(keypressed(const QString &)), this, SIGNAL(keypressed(const QString &)));
    connect(c, SIGNAL(leave(const QString&)), this, SIGNAL(leave(const QString&)));
    connect(c, SIGNAL(linkClicked(const QString &)), this, SLOT(on_linkClicked(const QString &)));
    
    return true;
}

bool ChatTab::delPrivate(const QString &name)
{
    return del(name);
}

QString ChatTab::getTopic(const QString &name)
{
    for (ChannelMap::iterator it = cmap.begin(); it != cmap.end(); it++)
	if (it.key().lower() == name.lower())
	    return it.data()->getTopic();
    return QString::null;
}

bool ChatTab::clearTopic(const QString &name)
{
    return setTopic(name, "");
}

bool ChatTab::setTopic(const QString &name, const QString &topic)
{
    for (ChannelMap::iterator it = cmap.begin(); it != cmap.end(); it++)
	if (it.key().lower() == name.lower())
	{
	    it.data()->setTopic(topic);
	    return true;
	}
    return false;
}

bool ChatTab::isBoard(const QString &name)
{
    ChannelMap::iterator it = cmap.find(name);
    return (it != cmap.end()) ? it.data()->isBoard() : false;  
}

bool ChatTab::writeToBoard(const QString &str)
{
    if (board)
    {
	board->append(str);
	return true;	
    }
    return false;
}

bool ChatTab::write(const QString &chn, const QString &str, QColor color)
{
    QString channel = chn;
    bool prvt = isPrivate(channel);
    if (isBoard(channel)) channel = MAIN_CHANNEL;
    for (ChannelMap::iterator it = cmap.begin(); it != cmap.end(); it++)
    {
	if ((!prvt && it.key().lower() == channel.lower()) || (prvt && it.key() == channel))
	{
	    QString header, footer;
	    header.sprintf("<font color=\"%s\">", static_cast<const char*>(color.name().utf8()));
	    footer = "</font>";
	    Channel *c = it.data();
	    c->append(header + str + footer);
	
	    if (c != currentPage())
	    {
		if (c->isPrivate())
		{
		    if (!c->getActivity()) 
			setTabIconSet(c, Icons::icon_private_alert);
		}
		else
		{
		    if (!c->getActivity())
			setTabIconSet(c, Icons::icon_channel_alert);
		}
		c->setActivity(true);
	    }
	    return true;
	}
    }
    return false;
}

void ChatTab::clearCurrent()
{
    currentPage()->clear();
}

QString ChatTab::channelsList()
{
    QString list;
    for (ChannelMap::iterator it = cmap.begin(); it != cmap.end(); it++ )
	if (!it.data()->isPrivate() && !it.data()->isBoard())
	    list.append(it.data()->name());
    return list + "#";
}

QStringList ChatTab::privatesList()
{
    QStringList list;
    for (ChannelMap::iterator it = cmap.begin(); it != cmap.end(); it++ )
	if (it.data()->isPrivate())
	    list.append(it.data()->name());
    return list;
}

void ChatTab::tabChanged(QWidget *widget)
{
    Channel *c = static_cast<Channel *>(widget);
    
    if (c->isPrivate())
    {
	if (c->getActivity()) 
	    setTabIconSet(c, Icons::icon_private);
    }
    else
    {
	if (c->getActivity())
	    if (c->isBoard())
		setTabIconSet(c, Icons::icon_board_normal);
	    else
		setTabIconSet(c, Icons::icon_channel_normal);
    }
    c->setActivity(false);
    
    emit saveChatLineText(lastactive);
    
    if (c->isBoard())
    {
	emit boardActivated();
    }
    else
    {
	if (isBoard(lastactive))
	    emit boardDisactivated();
	emit loadChatLineText(c->name());
	emit updateCaption();
    }
  
    lastactive = c->name();
}

bool ChatTab::clearPage(const QString &name)
{
    ChannelMap::iterator it = cmap.find(name);
    if (it != cmap.end())
    {
	Channel *c = it.data();
	c->clear();
	return true;
    }
    return false;
}

bool ChatTab::clearBoard()
{
    if (board)
    {
	board->clear();
	return true;
    }
    return false;
}

bool ChatTab::saveChatLine(const QString &chn, const QString &text)
{
    ChannelMap::iterator it = cmap.find(chn);
    if (it != cmap.end())
    {
	if (it.data()->setChatLineText(text))
	    return true;
    }
    return false;
}

QString ChatTab::loadChatLine(const QString &chn)
{
    ChannelMap::iterator it = cmap.find(chn);
    return (it == cmap.end()) ? QString::null : it.data()->getChatLineText();
}

void ChatTab::scrollDown(const QString &chn)
{
    ChannelMap::iterator it = cmap.find(chn);
    if (it != cmap.end()) it.data()->scrollToBottom();
}

void ChatTab::scrollDownCurrent()
{
    currentPage()->scrollToBottom();
}
